import { WidgetModel, IWidgetManager } from '@jupyter-widgets/base';
import ndarray = require('ndarray');
export declare type TypedArray = ndarray.TypedArray;
export declare type TypedArrayConstructor = Int8ArrayConstructor | Uint8ArrayConstructor | Int16ArrayConstructor | Uint16ArrayConstructor | Int32ArrayConstructor | Uint32ArrayConstructor | Uint8ClampedArrayConstructor | Float32ArrayConstructor | Float64ArrayConstructor;
export interface IArrayLookup {
    int8: Int8Array;
    int16: Int16Array;
    int32: Int32Array;
    uint8: Uint8Array;
    uint16: Uint16Array;
    uint32: Uint32Array;
    float32: Float32Array;
    float64: Float64Array;
}
/**
 * The serialized representation of a received array
 */
export interface IReceivedSerializedArray {
    shape: number[];
    dtype: keyof IArrayLookup;
    buffer: DataView;
}
/**
 * The serialized representation of a received, compressed array
 */
export interface IReceivedCompressedSerializedArray {
    shape: number[];
    dtype: keyof IArrayLookup;
    buffer?: DataView;
    compressed_buffer?: DataView;
}
/**
 * The serialized representation of an array for sending
 */
export interface ISendSerializedArray {
    shape: number[];
    dtype: keyof IArrayLookup;
    buffer: TypedArray | DataView;
}
/**
 * The serialized representation of a compressed array for sending
 */
export interface ISendCompressedSerializedArray {
    shape: number[];
    dtype: keyof IArrayLookup;
    buffer?: TypedArray | DataView;
    compressed_buffer?: TypedArray | DataView;
}
export declare type SendSerializedArray = ISendSerializedArray | ISendCompressedSerializedArray;
export declare function ensureSerializableDtype(dtype: ndarray.DataType): keyof IArrayLookup;
/**
 * Deserialize from JSON to an ndarray object.
 *
 * @param obj The deserialized JSON to convert
 * @param manager The owning widget manager
 *
 * @returns A new ndarray object.
 */
export declare function JSONToArray(obj: IReceivedSerializedArray | null, manager?: IWidgetManager): ndarray.NdArray | null;
/**
 * Serialize to JSON from an ndarray object.
 *
 * @param obj The ndarray object to convert
 * @param manager The owning widget model
 *
 * @returns The JSON object representing the ndarray.
 */
export declare function arrayToJSON(obj: ndarray.NdArray | null, widget?: WidgetModel): ISendSerializedArray | null;
/**
 * Serializers for to/from ndarrays
 */
export declare const array_serialization: {
    deserialize: typeof JSONToArray;
    serialize: typeof arrayToJSON;
};
export declare const typesToArray: {
    int8: Int8ArrayConstructor;
    int16: Int16ArrayConstructor;
    int32: Int32ArrayConstructor;
    uint8: Uint8ArrayConstructor;
    uint8_clamped: Uint8ClampedArrayConstructor;
    uint16: Uint16ArrayConstructor;
    uint32: Uint32ArrayConstructor;
    float32: Float32ArrayConstructor;
    float64: Float64ArrayConstructor;
};
export declare function compressedJSONToArray(obj: IReceivedCompressedSerializedArray | null, manager?: IWidgetManager): ndarray.NdArray | null;
export declare function arrayToCompressedJSON(obj: ndarray.NdArray | null, widget?: WidgetModel): SendSerializedArray | null;
export declare const compressed_array_serialization: {
    deserialize: typeof compressedJSONToArray;
    serialize: typeof arrayToCompressedJSON;
};
export declare function typedArrayToType(array: TypedArray): keyof IArrayLookup;
/**
 * Deserialize from JSON to a typed array. Discards shape information.
 *
 * @param obj The deserialized JSON to convert
 * @param manager The owning widget manager
 *
 * @returns A new typed array.
 */
export declare function JSONToTypedArray(obj: IReceivedSerializedArray | null, manager?: IWidgetManager): TypedArray | null;
/**
 * Serialize to JSON from a typed array.
 *
 * @param obj The typed array to convert
 * @param manager The owning widget model
 *
 * @returns The JSON object representing the typed array.
 */
export declare function typedArrayToJSON(obj: TypedArray | null, widget?: WidgetModel): ISendSerializedArray | null;
/**
 * Serializers for to/from 1D typed array
 */
export declare const typedarray_serialization: {
    deserialize: typeof JSONToTypedArray;
    serialize: typeof typedArrayToJSON;
};
export interface ISimpleObject {
    array: TypedArray;
    shape: number[];
}
/**
 * Deserialize from JSON to a simple object with shape and typed array.
 *
 * @param obj The deserialized JSON to convert
 * @param manager The owning widget manager
 *
 * @returns A new object containg the data.
 */
export declare function JSONToSimple(obj: IReceivedSerializedArray | null, manager?: IWidgetManager): ISimpleObject | null;
/**
 * Serialize to JSON from a simple object.
 *
 * @param obj The simple object to convert
 * @param manager The owning widget model
 *
 * @returns The JSON object representing the simple object.
 */
export declare function simpleToJSON(obj: ISimpleObject | null, widget?: WidgetModel): ISendSerializedArray | null;
/**
 * Serializers for to/from 1D simple object containing array and metadata.
 */
export declare const simplearray_serialization: {
    deserialize: typeof JSONToSimple;
    serialize: typeof simpleToJSON;
};
/**
 * Factory for serialziers to/from a typed array with fixed shape.
 */
export declare function fixed_shape_serialization(shape: number[]): {
    deserialize: (obj: IReceivedSerializedArray | null, manager?: IWidgetManager) => TypedArray | null;
    serialize: (obj: TypedArray | null, widget?: WidgetModel) => ISendSerializedArray | null;
};
//# sourceMappingURL=ndarray.d.ts.map