"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixed_shape_serialization = exports.simplearray_serialization = exports.simpleToJSON = exports.JSONToSimple = exports.typedarray_serialization = exports.typedArrayToJSON = exports.JSONToTypedArray = exports.typedArrayToType = exports.compressed_array_serialization = exports.arrayToCompressedJSON = exports.compressedJSONToArray = exports.typesToArray = exports.array_serialization = exports.arrayToJSON = exports.JSONToArray = exports.ensureSerializableDtype = void 0;
const util_1 = require("./util");
const compression_1 = require("./compression");
const ndarray = require("ndarray");
;
function ensureSerializableDtype(dtype) {
    if (dtype === 'array' || dtype === 'buffer' || dtype === 'generic' ||
        dtype === "bigint64" || dtype === "biguint64") {
        throw new Error(`Cannot serialize ndarray with dtype: ${dtype}.`);
    }
    else if (dtype === 'uint8_clamped') {
        dtype = 'uint8';
    }
    return dtype;
}
exports.ensureSerializableDtype = ensureSerializableDtype;
/**
 * Deserialize from JSON to an ndarray object.
 *
 * @param obj The deserialized JSON to convert
 * @param manager The owning widget manager
 *
 * @returns A new ndarray object.
 */
function JSONToArray(obj, manager) {
    if (obj === null) {
        return null;
    }
    // obj is {shape: list, dtype: string, array: DataView}
    // return an ndarray object
    return ndarray(new exports.typesToArray[obj.dtype](obj.buffer.buffer), obj.shape);
}
exports.JSONToArray = JSONToArray;
/**
 * Serialize to JSON from an ndarray object.
 *
 * @param obj The ndarray object to convert
 * @param manager The owning widget model
 *
 * @returns The JSON object representing the ndarray.
 */
function arrayToJSON(obj, widget) {
    if (obj === null) {
        return null;
    }
    let dtype = ensureSerializableDtype(obj.dtype);
    // serialize to {shape: list, dtype: string, array: buffer}
    return { shape: obj.shape, dtype, buffer: obj.data };
}
exports.arrayToJSON = arrayToJSON;
/**
 * Serializers for to/from ndarrays
 */
exports.array_serialization = { deserialize: JSONToArray, serialize: arrayToJSON };
exports.typesToArray = {
    int8: Int8Array,
    int16: Int16Array,
    int32: Int32Array,
    uint8: Uint8Array,
    uint8_clamped: Uint8ClampedArray,
    uint16: Uint16Array,
    uint32: Uint32Array,
    float32: Float32Array,
    float64: Float64Array
};
function compressedJSONToArray(obj, manager) {
    if (obj === null) {
        return null;
    }
    let buffer;
    if (obj.compressed_buffer !== undefined) {
        buffer = (0, compression_1.decompress)(obj.compressed_buffer.buffer);
    }
    else {
        buffer = obj.buffer.buffer;
    }
    // obj is {shape: list, dtype: string, array: DataView}
    // return an ndarray object
    return ndarray(new exports.typesToArray[obj.dtype](buffer), obj.shape);
}
exports.compressedJSONToArray = compressedJSONToArray;
function arrayToCompressedJSON(obj, widget) {
    if (obj === null) {
        return null;
    }
    let dtype = ensureSerializableDtype(obj.dtype);
    const level = widget ? widget.get('compression_level') : 0;
    if (level !== undefined && level > 0) {
        const compressed_buffer = (0, compression_1.compress)(obj.data.buffer, level);
        // serialize to {shape: list, dtype: string, array: buffer}
        return { shape: obj.shape, dtype, compressed_buffer };
    }
    // serialize to {shape: list, dtype: string, array: buffer}
    return { shape: obj.shape, dtype, buffer: obj.data };
}
exports.arrayToCompressedJSON = arrayToCompressedJSON;
exports.compressed_array_serialization = {
    deserialize: compressedJSONToArray,
    serialize: arrayToCompressedJSON
};
function typedArrayToType(array) {
    if (array instanceof Int8Array) {
        return 'int8';
    }
    else if (array instanceof Int16Array) {
        return 'int16';
    }
    else if (array instanceof Int32Array) {
        return 'int32';
    }
    else if (array instanceof Uint8Array) {
        return 'uint8';
    }
    else if (array instanceof Uint8ClampedArray) {
        return 'uint8';
    }
    else if (array instanceof Uint16Array) {
        return 'uint16';
    }
    else if (array instanceof Uint32Array) {
        return 'uint32';
    }
    else if (array instanceof Float32Array) {
        return 'float32';
    }
    else if (array instanceof Float64Array) {
        return 'float64';
    }
    else {
        throw new Error(`Unknown TypedArray type: ${array}`);
    }
}
exports.typedArrayToType = typedArrayToType;
;
/**
 * Deserialize from JSON to a typed array. Discards shape information.
 *
 * @param obj The deserialized JSON to convert
 * @param manager The owning widget manager
 *
 * @returns A new typed array.
 */
function JSONToTypedArray(obj, manager) {
    if (obj === null) {
        return null;
    }
    // obj is {shape: list, dtype: string, array: DataView}
    return new exports.typesToArray[obj.dtype](obj.buffer.buffer);
}
exports.JSONToTypedArray = JSONToTypedArray;
/**
 * Serialize to JSON from a typed array.
 *
 * @param obj The typed array to convert
 * @param manager The owning widget model
 *
 * @returns The JSON object representing the typed array.
 */
function typedArrayToJSON(obj, widget) {
    if (obj === null) {
        return null;
    }
    // serialize to {shape: list, dtype: string, array: buffer}
    return { shape: [obj.length], dtype: typedArrayToType(obj), buffer: obj };
}
exports.typedArrayToJSON = typedArrayToJSON;
/**
 * Serializers for to/from 1D typed array
 */
exports.typedarray_serialization = { deserialize: JSONToTypedArray, serialize: typedArrayToJSON };
/**
 * Deserialize from JSON to a simple object with shape and typed array.
 *
 * @param obj The deserialized JSON to convert
 * @param manager The owning widget manager
 *
 * @returns A new object containg the data.
 */
function JSONToSimple(obj, manager) {
    if (obj === null) {
        return null;
    }
    // obj is {shape: list, dtype: string, array: DataView}
    return { array: new exports.typesToArray[obj.dtype](obj.buffer.buffer), shape: obj.shape };
}
exports.JSONToSimple = JSONToSimple;
/**
 * Serialize to JSON from a simple object.
 *
 * @param obj The simple object to convert
 * @param manager The owning widget model
 *
 * @returns The JSON object representing the simple object.
 */
function simpleToJSON(obj, widget) {
    if (obj === null) {
        return null;
    }
    // serialize to {shape: list, dtype: string, array: buffer}
    return { shape: obj.shape, dtype: typedArrayToType(obj.array), buffer: obj.array };
}
exports.simpleToJSON = simpleToJSON;
/**
 * Serializers for to/from 1D simple object containing array and metadata.
 */
exports.simplearray_serialization = { deserialize: JSONToSimple, serialize: simpleToJSON };
/**
 * Factory for serialziers to/from a typed array with fixed shape.
 */
function fixed_shape_serialization(shape) {
    let fixedLength = 1;
    for (let dim of shape) {
        fixedLength *= dim;
    }
    function JSONToFixedShape(obj, manager) {
        if (obj === null) {
            return null;
        }
        if (!(0, util_1.arraysEqual)(obj.shape, shape)) {
            throw new Error(`Incoming data unexpected shape: ${obj.shape}, expected ${shape}`);
        }
        // obj is {shape: list, dtype: string, array: DataView}
        return new exports.typesToArray[obj.dtype](obj.buffer.buffer);
    }
    function fixedShapeToJSON(obj, widget) {
        if (obj === null) {
            return null;
        }
        if (obj.length !== fixedLength) {
            throw new Error(`Data has wrong size for fixed shape serialization! Expected ${fixedLength} elements, got ${obj.length}.`);
        }
        // serialize to {shape: list, dtype: string, array: buffer}
        return { shape, dtype: typedArrayToType(obj), buffer: obj };
    }
    return { deserialize: JSONToFixedShape, serialize: fixedShapeToJSON };
}
exports.fixed_shape_serialization = fixed_shape_serialization;
//# sourceMappingURL=ndarray.js.map