"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.data_union_simple_serialization = exports.JSONToSimpleUnion = exports.data_union_typedarray_serialization = exports.JSONToUnionTypedArray = exports.data_union_serialization = exports.data_union_array_serialization = exports.listenToUnion = exports.unionToJSON = exports.JSONToUnionArray = exports.JSONToUnion = void 0;
const base_1 = require("@jupyter-widgets/base");
const ndarray_1 = require("./ndarray");
// This is OK, as long as we only use it for type declarations
const common_1 = require("./common");
/**
 * Deserializes union JSON to an ndarray or a NDArrayModel, as appropriate.
 */
async function JSONToUnion(obj, manager) {
    if (typeof obj === 'string') {
        const modelPromise = (0, base_1.unpack_models)(obj, manager);
        return modelPromise;
    }
    else {
        return Promise.resolve((0, ndarray_1.JSONToArray)(obj, manager));
    }
}
exports.JSONToUnion = JSONToUnion;
/**
 * Deserializes union JSON to an ndarray, regardless of whether it is a widget reference or direct data.
 */
async function JSONToUnionArray(obj, manager) {
    return (0, common_1.getArray)(await JSONToUnion(obj, manager));
}
exports.JSONToUnionArray = JSONToUnionArray;
/**
 * Serializes a union to JSON.
 */
function unionToJSON(obj, widget) {
    if (obj instanceof base_1.WidgetModel) {
        return obj.toJSON(undefined);
    }
    else {
        return (0, ndarray_1.arrayToJSON)(obj, widget);
    }
}
exports.unionToJSON = unionToJSON;
/**
 * Sets up backbone events for listening to union changes.
 *
 * The callback will be called when:
 *  - The model is a widget, and its data changes
 *
 * Specify `allChanges` as truthy to also cover these cases:
 *  - The union changes from a widget to an array or vice-versa
 *  - The union is an array and its content changes
 *
 * To stop listening, call the return value.
 */
function listenToUnion(model, unionName, callback, allChanges) {
    function listenToWidgetChanges(union) {
        if (union instanceof base_1.WidgetModel) {
            // listen to changes in current model
            model.listenTo(union, 'change', callback);
        }
    }
    function onUnionChange(unionModel, value, subOptions) {
        const prev = model.previous(unionName) || [];
        const curr = value || [];
        if (prev instanceof base_1.WidgetModel) {
            model.stopListening(prev);
        }
        else if (allChanges && !(curr instanceof base_1.WidgetModel)) {
            // The union was an array, and has changed to a new array
            callback(unionModel, subOptions);
        }
        if (allChanges && (prev instanceof base_1.WidgetModel) !== (curr instanceof base_1.WidgetModel)) {
            // Union type has changed, call out
            callback(unionModel, subOptions);
        }
        listenToWidgetChanges(curr);
    }
    listenToWidgetChanges(model.get(unionName));
    // make sure to (un)hook listeners when property changes
    model.on('change:' + unionName, onUnionChange);
    function stopListening() {
        const curr = model.get(unionName);
        if (curr instanceof base_1.WidgetModel) {
            model.stopListening(curr);
        }
        model.off('change:' + unionName, onUnionChange);
    }
    return stopListening;
}
exports.listenToUnion = listenToUnion;
exports.data_union_array_serialization = {
    deserialize: JSONToUnionArray,
    serialize: unionToJSON
};
exports.data_union_serialization = {
    deserialize: JSONToUnion,
    serialize: unionToJSON
};
/**
 * Deserializes union JSON to an ndarray, regardless of whether it is a widget reference or direct data.
 */
function JSONToUnionTypedArray(obj, manager) {
    if (typeof obj === 'string') {
        const modelPromise = (0, base_1.unpack_models)(obj, manager);
        return modelPromise.then((model) => {
            const array = model.getNDArray();
            if (array === null) {
                return array;
            }
            return array.data;
        });
    }
    else {
        return Promise.resolve((0, ndarray_1.JSONToTypedArray)(obj, manager));
    }
}
exports.JSONToUnionTypedArray = JSONToUnionTypedArray;
exports.data_union_typedarray_serialization = {
    deserialize: JSONToUnionTypedArray,
    serialize: ndarray_1.typedArrayToJSON
};
/**
 * Deserializes union JSON to an ndarray, regardless of whether it is a widget reference or direct data.
 */
function JSONToSimpleUnion(obj, manager) {
    if (typeof obj === 'string') {
        const modelPromise = (0, base_1.unpack_models)(obj, manager);
        return modelPromise.then((model) => {
            const array = model.getNDArray();
            if (array === null) {
                return null;
            }
            return { array: array.data, shape: array.shape };
        });
    }
    else {
        return Promise.resolve((0, ndarray_1.JSONToSimple)(obj, manager));
    }
}
exports.JSONToSimpleUnion = JSONToSimpleUnion;
exports.data_union_simple_serialization = {
    deserialize: JSONToSimpleUnion,
    serialize: ndarray_1.simpleToJSON
};
//# sourceMappingURL=union.js.map